package net.nosial.socialclient.classes;

import net.nosial.socialclient.exceptions.ResolutionException;
import net.nosial.socialclient.objects.ResolvedServer;

import javax.naming.NamingException;
import javax.naming.directory.*;
import java.util.Hashtable;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class Resolver
{
    public static ResolvedServer resolveDomain(String domain) throws ResolutionException
    {
        final Pattern fullPattern = Pattern.compile("v=socialbox;sb-rpc=(https?://[^;]+);sb-key=([^;]+)");

        Hashtable<String, String> env = new Hashtable<>();
        env.put("com.sun.jndi.dns.timeout.retries", "1");
        env.put("com.sun.jndi.dns.cache.ttl", "1");
        env.put("com.sun.jndi.dns.cache.negative.ttl", "0");

        try
        {
            DirContext dirContext = new InitialDirContext(env);
            Attributes attributes = dirContext.getAttributes("dns:/" + domain, new String[]{"TXT"});
            Attribute attributeTXT = attributes.get("TXT");

            if (attributeTXT == null)
            {
                throw new ResolutionException("Failed to resolve DNS TXT records for " + domain);
            }

            StringBuilder fullRecordBuilder = new StringBuilder();

            for (int i = 0; i < attributeTXT.size(); i++)
            {
                String value = (String) attributeTXT.get(i);
                fullRecordBuilder.append(value.replaceAll("^\"|\"$", "").trim());
            }

            String fullRecord = fullRecordBuilder.toString();
            Matcher matcher = fullPattern.matcher(fullRecord);

            if (matcher.find())
            {
                String endpoint = matcher.group(1);
                String publicKey = matcher.group(2).replaceAll("\\s+", "");

                if (endpoint == null || endpoint.isEmpty())
                {
                    throw new ResolutionException("Failed to resolve RPC endpoint for " + domain);
                }

                if (publicKey == null || publicKey.isEmpty())
                {
                    throw new ResolutionException("Failed to resolve public key for " + domain);
                }

                return new ResolvedServer(endpoint, publicKey);
            }
            else
            {
                throw new ResolutionException("Failed to find valid SocialBox record for " + domain);
            }
        }
        catch (NamingException e)
        {
            throw new ResolutionException("Error resolving domain: " + e.getMessage(), e);
        }
    }
}