package net.nosial.socialclient.objects;

import net.nosial.socialclient.abstracts.RpcResult;
import net.nosial.socialclient.enums.StandardErrorCodes;

import java.util.Map;

public class RpcError extends RpcResult
{
    private final String id;
    private final String error;
    private final StandardErrorCodes code;

    /**
     * Constructs an instance of RpcError using data from the given map.
     *
     * @param data a map containing the data for the error. Must include keys "id" with a String value,
     *             "error" with a String value, and "code" with an integer value which represents the error code.
     */
    public RpcError(Map<String, Object> data)
    {
        super(false);
        this.id = (String) data.get("id");
        this.error = (String) data.get("error");
        this.code = StandardErrorCodes.resolveCode((int)data.get("code"));
    }

    /**
     * Retrieves the ID of the RPC error.
     *
     * @return the ID of the RPC error.
     */
    public String getId()
    {
        return this.id;
    }

    /**
     * Retrieves the error message associated with this RPC error.
     *
     * @return the error message as a String.
     */
    public String getError()
    {
        return this.error;
    }

    /**
     * Retrieves the standard error code associated with this RPC error.
     *
     * @return the standard error code.
     */
    public StandardErrorCodes getCode()
    {
        return this.code;
    }

    /**
     * Returns the current RpcError instance as the error response.
     *
     * @return the current RpcError instance.
     */
    @Override
    public RpcError getErrorResponse()
    {
        return this;
    }

    /**
     * Returns null as RpcError does not provide a successful response.
     *
     * @return null since RpcError instances do not represent successful responses.
     */
    @Override
    public RpcResponse getResponse()
    {
        return null;
    }
}
