package net.nosial.socialclient.objects;

import net.nosial.socialclient.abstracts.RpcResult;

import java.util.Map;

public class RpcResponse extends RpcResult
{
    private final String id;
    private final Object result;

    /**
     * Constructs an instance of RpcResponse using data from the given map.
     *
     * @param data a map containing the data for the response. Must include an "id" key with a String value
     *             and optionally a "result" key with any Object value.
     */
    public RpcResponse(Map<String, Object> data)
    {
        super(true);
        this.id = (String) data.get("id");
        this.result = data.get("result");
    }

    /**
     * Retrieves the ID of the RPC response.
     *
     * @return the ID of the RPC response.
     */
    public String getId()
    {
        return this.id;
    }

    /**
     * Retrieves the result object from the RPC response.
     *
     * @return the result object, which can be of any type
     */
    public Object getResult()
    {
        return this.result;
    }

    /**
     * Returns the error response for an RPC operation. For the
     * RpcResponse class, an error response is always null as
     * it's assumed to be successful with no errors.
     *
     * @return null as RpcResponse does not have an error response.
     */
    @Override
    public RpcError getErrorResponse()
    {
        return null;
    }

    /**
     * Retrieves the current RpcResponse instance.
     *
     * @return the current RpcResponse instance.
     */
    @Override
    public RpcResponse getResponse()
    {
        return this;
    }
}
