package net.nosial.socialclient.methods;

import net.nosial.socialclient.Client;
import net.nosial.socialclient.classes.Cryptography;
import net.nosial.socialclient.exceptions.CryptographyException;
import net.nosial.socialclient.exceptions.ResolutionException;
import net.nosial.socialclient.exceptions.RpcException;
import org.junit.jupiter.api.Test;

import java.security.KeyPair;

import static org.junit.jupiter.api.Assertions.*;

public class CreateSessionTest
{
    @Test
    void testSessionCreation()
    {
        Client socialClient;
        KeyPair keyPair;
        String sessionUuid;

        try
        {
            socialClient = new Client("n64.cc");
            keyPair = Cryptography.generateKeyPair();
        }
        catch (ResolutionException | CryptographyException e)
        {
            fail(e.getMessage(), e);
            return;
        }

        try
        {
            sessionUuid = socialClient.createSession(keyPair);
        }
        catch (RpcException e)
        {
            fail(e.getMessage(), e);
            return;
        }

        assertNotNull(sessionUuid);
        System.out.println("Session UUID: " + sessionUuid);
    }

    @Test
    public void pingSignedTest()
    {
        final Client socialClient;
        final KeyPair keyPair;
        final String sessionUuid;

        try
        {
            socialClient = new Client("n64.cc");
            keyPair = Cryptography.generateKeyPair();
            sessionUuid = socialClient.createAndSetSession(keyPair);

        }
        catch (ResolutionException | CryptographyException | RpcException e)
        {
            fail(e.getMessage(), e);
            return;
        }

        System.out.println("Session UUID: " + sessionUuid);
        assertNotNull(sessionUuid);

        try
        {
            assertTrue(socialClient.ping());
        }
        catch (RpcException e)
        {
            fail(e.getMessage(), e);
        }
    }
}
