package net.nosial.socialclient.methods;

import net.nosial.socialclient.Client;
import net.nosial.socialclient.classes.Cryptography;
import net.nosial.socialclient.exceptions.CryptographyException;
import net.nosial.socialclient.exceptions.ResolutionException;
import net.nosial.socialclient.exceptions.RpcException;
import net.nosial.socialclient.objects.standard.ImageCaptcha;
import net.nosial.socialclient.objects.standard.SelfUser;
import org.junit.jupiter.api.Test;

import javax.swing.*;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.security.KeyPair;
import java.util.Random;

import static org.junit.jupiter.api.Assertions.*;

public class RegisterTest
{
    @Test
    public void registerTest()
    {
        // Step 1. Connect and establish a session to the server
        final Client socialClient;
        final KeyPair keyPair;
        final String sessionUuid;

        try
        {
            socialClient = new Client("n64.cc");
            keyPair = Cryptography.generateKeyPair();
            sessionUuid = socialClient.createAndSetSession(keyPair);

        }
        catch (ResolutionException | CryptographyException | RpcException e)
        {
            fail(e.getMessage(), e);
            return;
        }

        System.out.println("Session UUID: " + sessionUuid);
        assertNotNull(sessionUuid);

        final SelfUser selfUser;
        Random random = new Random();
        String username = "user" + random.nextInt(10000);

        // Step 2. Register the username and getSelf
        try
        {
            assertTrue(socialClient.register(username));
            selfUser = socialClient.getMe();
        }
        catch (RpcException e)
        {
            fail(e.getMessage(), e);
            return;
        }

        assertNotNull(selfUser);

        // Step 3. Get the captcha
        ImageCaptcha captcha;

        try
        {
            captcha = socialClient.verificationGetImageCaptcha();
        }
        catch(RpcException e)
        {
            fail(e.getMessage(), e);
            return;
        }

        SwingUtilities.invokeLater(() -> {
            try
            {
                ImageFrame frame = new ImageFrame(captcha.getImageBuffer());
                frame.setVisible(true);
            }
            catch(IOException e)
            {
                e.printStackTrace();
            }
        });

        assertNotNull(captcha);

        // Step 4. Verify the captcha, get input from the user
        String captchaInput = JOptionPane.showInputDialog("Enter the captcha: ");
        assertNotNull(captchaInput);

        // Step 5. Verify the captcha
        try
        {
            assertTrue(socialClient.verificationAnswerImageCaptcha(captchaInput));
        }
        catch(RpcException e)
        {
            fail(e.getMessage(), e);
            return;
        }
    }

    private static class ImageFrame extends JFrame {
        public ImageFrame(BufferedImage image) {
            setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
            setResizable(false);

            JPanel panel = new JPanel() {
                @Override
                protected void paintComponent(Graphics g) {
                    super.paintComponent(g);
                    g.drawImage(image, 0, 0, null);
                }
            };

            panel.setPreferredSize(new Dimension(image.getWidth(), image.getHeight()));
            add(panel);
            pack();
            setLocationRelativeTo(null);
        }
    }
}
